<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Spatie\Permission\Models\Role;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('backend.user.index');
    }

    /**
     * Get the users
     */
    public function getUsers()
    {
        $users = User::with('roles')->latest()->get();

        return response()->json($users);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $roles = Role::all();

        return view('backend.user.create', compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $user = new User();

        $user->first_name_en = $request->first_name_en;
        $user->middle_name_en = $request->middle_name_en;
        $user->last_name_en = $request->last_name_en;
        $user->first_name_kh = $request->first_name_kh;
        $user->middle_name_kh = $request->middle_name_kh;
        $user->last_name_kh = $request->last_name_kh;
        $user->email = $request->email;
        $user->password = Hash::make($request->password);
        $user->role = 'admin';
        $user->sex = $request->sex;
        $user->dob = date('Y-m-d', strtotime($request->dob));
        $user->tel = $request->tel;
        $user->birth_place = $request->birth_place;
        $user->present_address = $request->present_address;

        if ($request->role) {
            $role = Role::where('id', $request->role)->first();

            if ($role) {
                $user->assignRole($role->name);
            }
        }

        if ($request->image) {
            $imageData = json_decode($request->image, true);
            if (isset($imageData['fileName'])) {
                $tempFileName = $imageData['fileName'];
                $tempPath = public_path('uploads/temp/' . $tempFileName);

                // Generate new file name
                $extension = pathinfo($tempFileName, PATHINFO_EXTENSION);
                $newFileName = 'avatar' . date('YmdHi') . '.' . $extension;
                $finalPath = public_path('uploads/avatars/' . $newFileName);

                if (file_exists($tempPath)) {
                    // Move and rename file to final path
                    if (rename($tempPath, $finalPath)) {
                        $user->image = $newFileName;
                    } else {
                        Log::error("Failed to move file from $tempPath to $finalPath");
                    }
                } else {
                    Log::error("Temporary file not found: " . $tempPath);
                }
            } else {
                Log::error("File name not found in image data: " . $request->image);
            }
        }

        $user->save();

        $notification = array(
            'status' => 'success',
            'message' => 'User created successfully.'
        );

        return redirect()->route('user')->with($notification);
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        $user = User::find($id);
        $roles = Role::all();

        return view('backend.user.edit', compact('user', 'roles'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        $user = User::find($id);

        $user->first_name_en = $request->first_name_en;
        $user->middle_name_en = $request->middle_name_en;
        $user->last_name_en = $request->last_name_en;
        $user->first_name_kh = $request->first_name_kh;
        $user->middle_name_kh = $request->middle_name_kh;
        $user->last_name_kh = $request->last_name_kh;
        $user->email = $request->email;
        $user->role = 'admin';
        $user->sex = $request->sex;
        $user->dob = date('Y-m-d', strtotime($request->dob));
        $user->tel = $request->tel;
        $user->birth_place = $request->birth_place;
        $user->present_address = $request->present_address;

        $user->roles()->detach();
        if ($request->role) {
            $role = Role::where('id', $request->role)->first();

            if ($role) {
                $user->assignRole($role->name);
            }
        }

        if ($request->image) {
            $imageData = json_decode($request->image, true);
            if (isset($imageData['fileName'])) {
                $tempFileName = $imageData['fileName'];
                $tempPath = public_path('uploads/temp/' . $tempFileName);

                // Generate new file name
                $extension = pathinfo($tempFileName, PATHINFO_EXTENSION);
                $newFileName = 'avatar' . date('YmdHi') . '.' . $extension;
                $finalPath = public_path('uploads/avatars/' . $newFileName);

                // Check and delete existing file
                if ($user->image) {
                    $existingFilePath = public_path('uploads/avatars/' . $user->image);
                    if (File::exists($existingFilePath)) {
                        File::delete($existingFilePath);
                    }
                }

                if (file_exists($tempPath)) {
                    // Move and rename file to final path
                    if (rename($tempPath, $finalPath)) {
                        $user->photo = $newFileName;
                    } else {
                        Log::error("Failed to move file from $tempPath to $finalPath");
                    }
                } else {
                    Log::error("Temporary file not found: " . $tempPath);
                }
            } else {
                Log::error("File name not found in image data: " . $request->image);
            }
        }

        $user->save();

        $notification = array(
            'status' => 'success',
            'message' => 'User updated successfully.'
        );

        return redirect()->route('user')->with($notification);
    }

    /**
     * Update the specified resource in storage.
     */
    public function status(Request $request)
    {
        $user = User::find($request->id);

        $user->activation = $request->activation;
        $user->save();

        $notification = array(
            'status' => 'success',
            'message' => 'User status updated successfully.'
        );

        return response()->json($notification);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $user = User::find($id);

        if (!is_null($user)) {
            $user->delete();
        }

        $notification = array(
            'status' => 'success',
            'message' => 'User deleted successfully.'
        );

        return response()->json($notification);
    }
}
